/***************************************************************************
 *   Copyright (C) 2008 by Patrick Steiner <patrick.steiner@a1.net>        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <QtGui>
#include <QtDBus>

#include "btsearchdlg.h"
#include "app.h"

BtSearchDialog::BtSearchDialog(DeviceListEntry *_btDevice, QWidget *parent): QDialog(parent)
{
	Q_ASSERT(_btDevice);
    btDevice = _btDevice;

	btDevice->address = "00:00:00:00:00:00";
	btDevice->name = "N/A";

	setupUi(this);

	btnOK->setEnabled(false);

	connect(btnOK, SIGNAL(clicked()),
		this, SLOT(okClicked()));

	connect(btnBtSearch, SIGNAL(clicked()),
		this, SLOT(searchClicked()));

	connect(lstBtDevices, SIGNAL(itemSelectionChanged()),
		this, SLOT(selectionChanged()));

	this->setWindowTitle(tr("Configure Bluetooth - %1").arg(APP_NAME));
}

BtSearchDialog::~BtSearchDialog()
{
	// if i enable this the app segfaults, if i only open the dlg an close it again.
	/*
	delete bluetoothManager;
	delete bluetoothAdapter;
	*/
}

void BtSearchDialog::okClicked()
{
	QString currentBtName;

	if(lstBtDevices->count() > 0) {
		currentBtName = lstBtDevices->currentItem()->text();
	}

	if(!btDevicesList.isEmpty()) {
		for (int i = 0; i < btDevicesList.size(); ++i) {
			if (btDevicesList.at(i).name == currentBtName) {
				//qDebug() << "dbg(): Adding device:" << btDevicesList.at(i).address << btDevicesList.at(i).name;

				btDevice->address = btDevicesList.at(i).address;
				btDevice->name = btDevicesList.at(i).name;
			}
 		}
	}
}

void BtSearchDialog::searchClicked()
{
	btSeachDialogDBusInit();
}

void BtSearchDialog::btSeachDialogDBusInit()
{
	btnBtSearch->setEnabled(false);

	//BlueZ API: http://bluez.cvs.sourceforge.net/*checkout*/bluez/utils/hcid/dbus-api.txt
	bluetoothManager = new QDBusInterface("org.bluez","/org/bluez","org.bluez.Manager",QDBusConnection::systemBus());
	if (!bluetoothManager->isValid()) {
		//TODO: add errorbox
		qCritical() << "Unable to connect to the bluez dbus-service: " +
			bluetoothManager->lastError().name() +
			": "
			+ bluetoothManager->lastError().message();
		return;
	} else {
		qDebug() << "dbg(): bluez dbus-service found";
	}

	QDBusReply<QString> adapter = bluetoothManager->call("DefaultAdapter");
	if (!adapter.isValid()) {
		//TODO: add errorbox
		qCritical() << "No bluetooth adapter found";
		return;
	} else {
		qDebug() << "dbg(): bluetooth adapter found:" << adapter;
	}

	bluetoothAdapter = new QDBusInterface("org.bluez",adapter.value(),"org.bluez.Adapter",QDBusConnection::systemBus());
	if (!bluetoothAdapter->isValid()) {
		//TODO: add errorbox
		qCritical() << "Error creating adapter (DBUS):" +
			bluetoothAdapter->lastError().name() +
			": " +
			bluetoothAdapter->lastError().message();
                return;
        } else {
		qDebug() << "dbg(): Using adapter:" << adapter.value();
	}

	connect(bluetoothAdapter, SIGNAL(RemoteNameUpdated(const QString&, const QString&)),
		this, SLOT(slotRemoteNameUpdated(const QString&, const QString&)));

	connect(bluetoothAdapter, SIGNAL(DiscoveryCompleted()),
		this, SLOT(slotDiscoveryCompleted()));

	bluetoothAdapter->call("DiscoverDevices");
	bluetoothAdapter->call("DiscoveryCompleted");
}

void BtSearchDialog::slotRemoteNameUpdated(const QString &address, const QString &name)
{
	DeviceListEntry btEntry;

	qDebug() << "dbg(): bluetooth device found:" << address << "(" << name << ")";
	btEntry.address = address;
	btEntry.name = name;
	btDevicesList.append(btEntry);

	QListWidgetItem *btDevice = new QListWidgetItem;
	btDevice->setText(name);

	lstBtDevices->addItem(btDevice);
}

void BtSearchDialog::slotDiscoveryCompleted()
{
	qDebug() << "dbg(): bluetooth search finished";

	btnBtSearch->setEnabled(true);
}

void BtSearchDialog::selectionChanged()
{
	if(lstBtDevices->currentItem()->isSelected()) {
		btnOK->setEnabled(true);
	} else {
		btnOK->setEnabled(false);
	}
}

