/***************************************************************************
 *   Copyright (C) 2005-2007 by Patrick Steiner                            *
 *   patrick.steiner@a1.net                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <QApplication>
#include <QtGui>

#include "smssendwindow.h"
#include "app.h"

QIcon *Icon_AppIcon;
QString appPath;

QString locateFile(QString fileType, const QString &filename)
{
	QString filePath;

	if(fileType == "data") {
		filePath = appPath + "/../share/past/icons/" + filename;
	}

	if(!QFile::exists(filePath)) {
		QMessageBox::critical(0, QObject::tr("Systray"),
			QObject::tr("Couldn't find %1 "
				"on this system.").arg(filename));
		exit(1);
	}

	//qDebug() << "found file:" << filePath;

	return filePath;
}

int main(int argc, char *argv[])
{
	QApplication *app = new QApplication(argc, argv);

	appPath = app->applicationDirPath();

	if (!QSystemTrayIcon::isSystemTrayAvailable()) {
		QMessageBox::critical(0, QObject::tr("Systray"),
			QObject::tr("Couldn't detect any system tray "
				"on this system."));
		return 1;
	}

	//TODO: add linux like path

	QString foo = locateFile("data", "past.png");

	Icon_AppIcon = new QIcon(locateFile("data", "past.png"));

	QDesktopWidget *Desktop = QApplication::desktop();
	SmsSendWindow *sendWindow = new SmsSendWindow();

	// center the window
	int x = (Desktop->width()-sendWindow->width())/2;
	int y = (Desktop->height()-sendWindow->height())/2;
	sendWindow->move(x,y);

	sendWindow->setWindowTitle(QObject::tr("Send SMS - %1").arg(APP_NAME));
	sendWindow->setWindowIcon(*Icon_AppIcon);

	bool startMinimized = false;
	QSettings settings("past");
	startMinimized = settings.value("startminimized").toBool();
	if(startMinimized) {
		sendWindow->hide();
	} else {
		sendWindow->show();
	}

	app->setQuitOnLastWindowClosed(false);

	return app->exec();
}
