/***************************************************************************
 *   Copyright (C) 2005-2007 by Patrick Steiner                            *
 *   patrick.steiner@a1.net                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <QWidget>
#include <QDebug>
#include <QFile>
#include <QDir>
#include "phonebook.h"

PhoneBook::PhoneBook(QWidget *parent): QWidget(parent)
{
	//TODO: use better location
	//KDE 3 standard adressbook file
	kabcBook = QDir::homePath() + QDir::separator() +
				".kde" + QDir::separator() +
				"share" + QDir::separator() +
				"apps" + QDir::separator() +
				"kabc" + QDir::separator() +
				"std.vcf";

	// Past stanard adressbook file
	pastBook = QDir::homePath() + QDir::separator() +
				"past-addressbook.txt";

	QFileInfo i_kabc(kabcBook);
	QFileInfo i_past(pastBook);

	//TODO: include settings option to switch manual
	if(QFile::exists(kabcBook) && i_kabc.size() > 0) {
		qDebug() << "dbg() found kaddressbook and use it...";
		readContactsFromKABC(kabcBook);
	} else if(QFile::exists(pastBook) && i_past.size() > 0) {
		qDebug() << "dbg() found past-addressbook and use it...";
		readContactsFromFile(pastBook);
	} else {
		qWarning("No useable addressbook for autocompletion found");
	}
}

void PhoneBook::readContactsFromFile(QString aBookFile)
{
	QFile aBook(aBookFile);

	if(!aBook.open(QIODevice::ReadOnly|QIODevice::Text)) {
		qWarning() << "ERROR:"
			   << __FUNCTION__
			   << aBookFile
			   << "does not exist";
		return;
	}

	while(!aBook.atEnd()) {
		QByteArray line = aBook.readLine();
		if(!line.trimmed().isEmpty()) {
			QString b = line.trimmed();
			QStringList aEntry = b.split(",");

			ContactEntry cEntry;
			cEntry.Name = aEntry[0].trimmed();
			cEntry.Phonenumber = aEntry[1].trimmed();

			//qDebug() << "dbg() name" << cEntry.Name << "number" << cEntry.Phonenumber;

			addressBook.append(cEntry);
		}
	}
	//qDebug() << "dbg() entries count:" << addressBook.count();

	aBook.close();
}

void PhoneBook::readContactsFromKABC(QString aBookFile)
{
	QString vcdName;
	QString vcdMobile;

	QFile aBook(aBookFile);

	if(!aBook.open(QIODevice::ReadOnly|QIODevice::Text)) {
		qWarning() << "ERROR:"
			   << __FUNCTION__
			   << aBookFile
			   << "does not exist";
		return;
	}
	while(!aBook.atEnd()) {
		QByteArray line = aBook.readLine();

		if(!line.trimmed().isEmpty()) {
			
			if(line.startsWith("BEGIN:VCARD")) {
				vcdName = QString::null;
				vcdMobile = QString::null;
			}

			if(line.startsWith("N:")) {
				vcdName = getVCardName(line.trimmed());
			}

			if(line.startsWith("TEL;TYPE=CELL:")) {
				vcdMobile = getVCardPhonenumber(line.trimmed());
			}

			if(line.startsWith("END:VCARD") && !vcdName.isEmpty() && !vcdMobile.isEmpty()) {
				ContactEntry cEntry;
				cEntry.Name = vcdName;
				cEntry.Phonenumber = vcdMobile;
				addressBook.append(cEntry);

				//qDebug() << "dbg() name" << cEntry.Name << "number" << cEntry.Phonenumber;
			}
		}
	}
	//qDebug() << "dbg() entries count:" << addressBook.count();
	
	aBook.close();
}

QStringList PhoneBook::getAllContacts()
{
	int i;
	QStringList contactList;

	for(i=0; i<addressBook.count(); i++) {
		contactList << addressBook.at(i).Name;
	}

	return contactList;
}

QString PhoneBook::getPhoneNumberByName(QString Name)
{
	int i;
	QString pNumber;

	for(i=0; i<addressBook.count(); i++) {
		if(Name == addressBook.at(i).Name) {
			pNumber = addressBook.at(i).Phonenumber;
		}
	}

	if(!pNumber.isEmpty()) {
		return pNumber;
	} else {
		qWarning() << "ERROR:" << __FUNCTION__ << "cannot find phonenumber";
		return QString::null;
	}
}

QString PhoneBook::getVCardName(QString VcardLine)
{
	QString name;

	// remove the N from the string
	QStringList lineSplit = VcardLine.split(":");

	// split givenname and familyname, sperator is ;
	QStringList nameSplit = lineSplit[1].split(";");

	// build a name givenname and familyname
	name = nameSplit[1] + " " + nameSplit[0];

	return name;
}

QString PhoneBook::getVCardPhonenumber(QString VcardLine)
{
	QString phonenumber;

	QStringList lineSplit = VcardLine.split(":");

	phonenumber = lineSplit[1];

	return phonenumber;
}

// vim: ts=4
