/***************************************************************************
 *   Copyright (C) 2007 by Patrick Steiner <patrick.steiner@a1.net>        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <QtGui>
#include "settingsdlg.h"
#include "app.h"
#include "btsearchdlg.h"

SettingsDialog::SettingsDialog(QWidget *parent): QDialog(parent)
{
	d_storesmspath = QDir::homePath() + QDir::separator() + "past-messages.txt";

	settings = new QSettings("past");

	setupUi(this);

	readSettings();

	connect(btnCancel, SIGNAL(clicked()),
		this, SLOT(close()));

	connect(btnOK, SIGNAL(clicked()),
		this, SLOT(okClicked()));

	connect(radioBluetooth, SIGNAL(clicked()),
		this, SLOT(bluetoothClicked()));

	connect(radioOther, SIGNAL(clicked()),
		this, SLOT(otherClicked()));

	connect(chkBxStoreSMS, SIGNAL(clicked()),
		this, SLOT(storeSMSClicked()));

	connect(btnSMSStorePath, SIGNAL(clicked()),
		this, SLOT(storeSMSPathClicked()));

	connect(btnBtSearch, SIGNAL(clicked()),
		this, SLOT(searchBluetoothDevice()));

	this->setWindowTitle(tr("Configure - %1").arg(APP_NAME));

	this->show();
}

void SettingsDialog::readSettings()
{
	// general settings
	s_startminimized = settings->value("startminimized").toBool();
	s_multisms = settings->value("multisms").toBool();
	s_storesms = settings->value("storesms").toBool();
	s_storesmspath = settings->value("storesmspath").toString();

	// phone settings
	s_phonePort = settings->value("phonePort").toString();
	s_phoneModel = settings->value("phoneModel").toString();
	s_phoneConnectionType = settings->value("phoneConnectionType").toString();
	s_phoneRfChannel = settings->value("phoneRfChannel").toInt();

	if(s_startminimized)
		chkBxStartMinimized->setChecked(true);

	if(s_multisms)
		chkBxMultiSMS->setChecked(true);

	if(s_storesms) {
		chkBxStoreSMS->setChecked(true);
		lnSMSStorePath->setEnabled(true);
		btnSMSStorePath->setEnabled(true);
		lblSMSStorePath->setEnabled(true);

		if(lnSMSStorePath->text().isEmpty()) {
			lnSMSStorePath->setText(d_storesmspath);
		}
	} else {
		chkBxStoreSMS->setChecked(false);
		lnSMSStorePath->setEnabled(false);
		btnSMSStorePath->setEnabled(false);
		lblSMSStorePath->setEnabled(false);
	}

	lnSMSStorePath->setText(s_storesmspath);

	if(s_phoneConnectionType == "bluetooth") {
		radioBluetooth->setChecked(true);
		radioOther->setChecked(false);
		lnBluetoothAddress->setEnabled(true);
		cmbPhonePort->setEnabled(false);
		btnBtSearch->setEnabled(true);
		spnBxRf_Channel->setEnabled(true);
		spnBxRf_Channel->setValue(s_phoneRfChannel);

		lnBluetoothAddress->setText(s_phonePort);
	} else {
		radioBluetooth->setChecked(false);
		btnBtSearch->setEnabled(false);
		spnBxRf_Channel->setEnabled(false);
		radioOther->setChecked(true);
		lnBluetoothAddress->setEnabled(false);
		cmbPhonePort->setEnabled(true);

		QStringList phonePort;
		//TODO: why QDir::separator() doesnt work?
		QString devicePath = "/dev/";

		//add standard ports
		phonePort 	<< "rfcomm0"
				<< "rfcomm1"
				<< "irda0"
				<< "ttyS0"
				<< "modem";

		int i;

		cmbPhonePort->addItem(s_phonePort);

		for(i=0; i<phonePort.count(); i++) {
			QString deviceP = devicePath + phonePort[i];

			if(QFile::exists(deviceP) && QString::compare(deviceP, s_phonePort, Qt::CaseSensitive)) {
				cmbPhonePort->addItem(deviceP);
			}
		}
	}

	lnPhoneModel->setText(s_phoneModel);
}

void SettingsDialog::writeSettings()
{
	settings->setValue("startminimized", chkBxStartMinimized->isChecked());
	settings->setValue("multisms", chkBxMultiSMS->isChecked());

	settings->setValue("phoneModel", lnPhoneModel->text());


	settings->setValue("storesms", chkBxStoreSMS->isChecked());

	if(chkBxStoreSMS->isChecked()) {
		settings->setValue("storesmspath", lnSMSStorePath->text());
	}

	if(radioBluetooth->isChecked()) {
		settings->setValue("phoneConnectionType", "bluetooth");
		settings->setValue("phonePort", lnBluetoothAddress->text());
		settings->setValue("phoneRfChannel", spnBxRf_Channel->value());
	} else if(radioOther->isChecked()) {
		settings->setValue("phoneConnectionType", "serial");
		settings->setValue("phonePort", cmbPhonePort->currentText());
	} else {
		qWarning() << "ERROR:" << __FUNCTION__ << "no connection type found";
	}
}

void SettingsDialog::okClicked()
{
	if(chkBxStoreSMS->isChecked() && lnSMSStorePath->text().isEmpty()) {
		QMessageBox::critical(this,
			tr("%1 - Error").arg(APP_NAME),
			tr("Please select a SMS Store Path"));
		return;
	}

	writeSettings();
	this->close();
}

void SettingsDialog::bluetoothClicked()
{
	lnBluetoothAddress->setEnabled(true);
	cmbPhonePort->setEnabled(false);
	btnBtSearch->setEnabled(true);
	spnBxRf_Channel->setEnabled(true);
}

void SettingsDialog::otherClicked()
{
	lnBluetoothAddress->setEnabled(false);
	cmbPhonePort->setEnabled(true);
	btnBtSearch->setEnabled(false);
	spnBxRf_Channel->setEnabled(false);
}

void SettingsDialog::storeSMSClicked()
{
	if(chkBxStoreSMS->isChecked()) {
		lnSMSStorePath->setEnabled(true);
		btnSMSStorePath->setEnabled(true);
		lblSMSStorePath->setEnabled(true);

		if(lnSMSStorePath->text().isEmpty()) {
			lnSMSStorePath->setText(d_storesmspath);
		}
	} else {
		lnSMSStorePath->setEnabled(false);
		btnSMSStorePath->setEnabled(false);
		lblSMSStorePath->setEnabled(false);
	}
}

void SettingsDialog::storeSMSPathClicked()
{
	QString savePath = QFileDialog::getSaveFileName(this, tr("%1 - Store Messages to...").arg(APP_NAME),
				QDir::homePath(),
				tr("(*.txt)"),0,
				QFileDialog::DontConfirmOverwrite);

	
	lnSMSStorePath->setText(savePath);
}

void SettingsDialog::searchBluetoothDevice()
{
	DeviceListEntry btDevice;

	BtSearchDialog dlg(&btDevice, this);

	if(dlg.exec() == 1) {
		qDebug() << "dbg(): btaddress:" << btDevice.address;
		qDebug() << "dbg(): btname:" << btDevice.name;

		lnBluetoothAddress->setText(btDevice.address);
	}
}

